"""Helper Methods for testing."""

from contextlib import contextmanager


def eq_(one, other):
    assert one == other, f"{one} != {other}"


# hack own assert_raises, because py26 has a different implementation
@contextmanager
def assert_raises(exccls, msg):
    """Check exception of class `exccls` to be raised with message `msg`."""
    try:
        yield
        raise AssertionError(f"{exccls!r} not raised")
    except Exception as exc:
        assert isinstance(exc, exccls), f"{exc!r} is not a {exccls!r}"
        eq_(str(exc), msg)



"""
Node Classes.

* :any:`AnyNode`: a generic tree node with any number of attributes.
* :any:`Node`: a simple tree node with at least a name attribute and any number of additional attributes.
* :any:`NodeMixin`: extends any python class to a tree node.
* :any:`SymlinkNode`: Tree node which references to another tree node.
* :any:`SymlinkNodeMixin`: extends any Python class to a symbolic link to a tree node.
* :any:`LightNodeMixin`: A :any:`NodeMixin` using slots.
"""

from .anynode import AnyNode
from .exceptions import LoopError, TreeError
from .lightnodemixin import LightNodeMixin
from .node import Node
from .nodemixin import NodeMixin
from .symlinknode import SymlinkNode
from .symlinknodemixin import SymlinkNodeMixin

__all__ = [
    "AnyNode",
    "LightNodeMixin",
    "LoopError",
    "Node",
    "NodeMixin",
    "SymlinkNode",
    "SymlinkNodeMixin",
    "TreeError",
]



"""Exporter."""

from .dictexporter import DictExporter
from .dotexporter import DotExporter, UniqueDotExporter
from .jsonexporter import JsonExporter
from .mermaidexporter import MermaidExporter

__all__ = [
    "DictExporter",
    "DotExporter",
    "JsonExporter",
    "MermaidExporter",
    "UniqueDotExporter",
]



"""Powerful and Lightweight Python Tree Data Structure."""

__version__ = "2.12.1"
__author__ = "c0fec0de"
__author_email__ = "c0fec0de@gmail.com"
__description__ = """Powerful and Lightweight Python Tree Data Structure."""
__url__ = "https://github.com/c0fec0de/anytree"

from . import cachedsearch, util
from .iterators import LevelOrderGroupIter, LevelOrderIter, PostOrderIter, PreOrderIter, ZigZagGroupIter
from .node import AnyNode, LightNodeMixin, LoopError, Node, NodeMixin, SymlinkNode, SymlinkNodeMixin, TreeError
from .render import AbstractStyle, AsciiStyle, ContRoundStyle, ContStyle, DoubleStyle, RenderTree
from .resolver import ChildResolverError, Resolver, ResolverError, RootResolverError
from .search import CountError, find, find_by_attr, findall, findall_by_attr
from .walker import Walker, WalkError

# legacy
LevelGroupOrderIter = LevelOrderGroupIter


__all__ = [
    "AbstractStyle",
    "AnyNode",
    "AsciiStyle",
    "ChildResolverError",
    "ContRoundStyle",
    "ContStyle",
    "CountError",
    "DoubleStyle",
    "LevelGroupOrderIter",
    "LevelOrderGroupIter",
    "LevelOrderIter",
    "LightNodeMixin",
    "LoopError",
    "Node",
    "NodeMixin",
    "PostOrderIter",
    "PreOrderIter",
    "RenderTree",
    "Resolver",
    "ResolverError",
    "RootResolverError",
    "SymlinkNode",
    "SymlinkNodeMixin",
    "TreeError",
    "WalkError",
    "Walker",
    "ZigZagGroupIter",
    "cachedsearch",
    "find",
    "find_by_attr",
    "findall",
    "findall_by_attr",
    "util",
]



"""
Tree Iteration.

* :any:`PreOrderIter`: iterate over tree using pre-order strategy (self, children)
* :any:`PostOrderIter`: iterate over tree using post-order strategy (children, self)
* :any:`LevelOrderIter`: iterate over tree using level-order strategy
* :any:`LevelOrderGroupIter`: iterate over tree using level-order strategy returning group for every level
* :any:`ZigZagGroupIter`: iterate over tree using level-order strategy returning group for every level
"""

from .abstractiter import AbstractIter
from .levelordergroupiter import LevelOrderGroupIter
from .levelorderiter import LevelOrderIter
from .postorderiter import PostOrderIter
from .preorderiter import PreOrderIter
from .zigzaggroupiter import ZigZagGroupIter

__all__ = [
    "AbstractIter",
    "LevelOrderGroupIter",
    "LevelOrderIter",
    "PostOrderIter",
    "PreOrderIter",
    "ZigZagGroupIter",
]



from shutil import which

from pytest import mark
from test2ref import assert_refdata

from anytree import Node
from anytree.dotexport import RenderTreeGraph


def test_tree1(tmp_path):
    """Tree1."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    Node("sub0B", parent=s0)
    Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root)
    Node("sub1A", parent=s1)
    Node("sub1B", parent=s1)
    s1c = Node("sub1C", parent=s1)
    Node(99, parent=s1c)

    RenderTreeGraph(root).to_dotfile(tmp_path / "tree1.dot")
    assert_refdata(test_tree1, tmp_path)


def test_tree2(tmp_path):
    """Tree2."""
    root = Node("root")
    s0 = Node("sub0", parent=root, edge=2)
    Node("sub0B", parent=s0, foo=4, edge=109)
    Node("sub0A", parent=s0, edge="")
    s1 = Node("sub1", parent=root, edge="")
    Node("sub1A", parent=s1, edge=7)
    Node("sub1B", parent=s1, edge=8)
    s1c = Node("sub1C", parent=s1, edge=22)
    Node("sub1Ca", parent=s1c, edge=42)

    def nodenamefunc(node):
        return f"{node.name}:{node.depth}"

    def edgeattrfunc(node, child):
        return f'label="{node.name}:{child.name}"'

    r = RenderTreeGraph(
        root,
        options=["rankdir=LR;"],
        nodenamefunc=nodenamefunc,
        nodeattrfunc=lambda node: "shape=box",
        edgeattrfunc=edgeattrfunc,
    )

    r.to_dotfile(tmp_path / "tree2.dot")
    assert_refdata(test_tree2, tmp_path)


@mark.skipif(which("dot") is None, reason="requires graphviz`s `dot` command")
def test_tree_png(tmp_path):
    """Tree to png."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    Node("sub0B", parent=s0)
    Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root)
    Node("sub1A", parent=s1)
    Node("sub1B", parent=s1)
    s1c = Node("sub1C", parent=s1)
    Node("sub1Ca", parent=s1c)

    RenderTreeGraph(root).to_picture(tmp_path / "tree1.png")
